(* $Id: printer.ml,v 1.5 2005/05/25 01:04:17 garrigue Exp $ *)

open StdLabels
open Terms
open Format

let rec count_nil n = function
    [] -> (n, [])
  | Nil :: l -> count_nil (n+1) l
  | l -> (n,l)

let rec flatten_cons acc = function
    Cons (a, l) -> flatten_cons (a::acc) l
  | Nil ->
      let (n, l) = count_nil 0 acc in (List.rev l, Nil, n)
  | r -> (List.rev acc, r, 0)

let rec print_val ppf = function
    Nil -> fprintf ppf "nil"
  | Atom s | Var s -> fprintf ppf "%s" s
  | Cons (a,l) ->
      let (l, r, n) = flatten_cons [] l in
      if l = [] && a = Nil && r = Nil then
        fprintf ppf "%d" (n + 1)
      else
        fprintf ppf "@[<1>(%a%t%t)@]"
          print_val a
          (fun ppf ->
            List.iter l ~f:(fun a -> fprintf ppf "@ %a" print_val a)) 
          (fun ppf ->
	    if r <> Nil then fprintf ppf ".@;<0>%a" print_val r
	    else if n > 0 then fprintf ppf ".%d" n)

let print_expr ppf = function
    Cons(Atom("cons"|"=?" as s),Cons(e1,Cons(e2,Nil)))->
      fprintf ppf "@[<2>%s@ %a@ %a@]" s
	print_val e1 print_val e2
  | Cons(Atom("hd"|"tl" as s), Cons(e, Nil)) ->
      fprintf ppf "@[<2>%s@ %a@]" s print_val e
  | e ->
      print_val ppf e

let rec print_cmd ppf = function
    Assign(x,e) ->
      fprintf ppf "@[<2>%s :=@ %a@]" x print_expr e
  | Seq(c1, c2) ->
      fprintf ppf "%a;@ %a" print_cmd c1 print_cmd c2
  | Skip ->
      fprintf ppf "{}"
  | While (e, c) ->
      fprintf ppf "@[<2>@[while@;<1 2>%a@ do@]%a@]"
	print_expr e
	(fun ppf -> function
	    Seq _ -> fprintf ppf " {@ @[%a@]@ }" print_cmd c
	  | _ -> fprintf ppf "@ %a" print_cmd c)
	c
  | Call (x,f,e) ->
      fprintf ppf "@[<2>%s :=@ %s@ %a@]" x f print_expr e

